import requests
import json
import yaml

def callServer(method: str, url: str, host: str, headers: dict, params: dict):
    try:
        response = requests.request(method=method, url=url, headers=headers, params=params)
    except requests.ConnectionError as e:
        print("Error connecting to Joule server: {}".format(host))
        exit(1)
    return response


def postData(url: str, filename: str, params: dict, host: str, headers: dict):
    with open(filename) as json_data:

        ext = filename.split('.')[1].lower()
        if ext == "json":
            try:
                data = json.load(json_data)
            except json.decoder.JSONDecodeError as e:
                print("Error parsing JSON file")
                exit(1)
        elif ext == 'yaml' or ext == 'yml':
            try:
                data = yaml.load(json_data, Loader=yaml.FullLoader)
            except yaml.YAMLError as exc:
                print("Error parsing YAML file")
                exit(1)
        else:
            print("Invalid file type: {}".format(ext))
            exit(1)

        try:
            response = requests.post(url, headers=headers, json=data, params=params)
        except requests.ConnectionError as e:
            print("Error connecting to Joule server: {}".format(host))
            exit(1)
    return response


import click
import requests
from tabulate import tabulate

def displayResponse(name: str,response: requests.Response):
    table = []
    if response.status_code != 204:
        data = response.json()
        cmd = data["command"]
        code = data["aggregatedServiceCode"]
        table.append([name,cmd,code])
        click.echo(tabulate(table, headers=["Name", "Command","Response"]))
    else:
        click.echo("Unknown use case - {}".format(name))

@click.command()
@click.pass_context
@click.argument("name")
@click.option("--pod", help="Joule pod name", default="")
def resume(ctx: click.Context, name: str, pod: str):
    """Resume use case processing

    NAME is the use case
    """
    joule_host = ctx.obj["joule_host"]
    url = f"http://{joule_host}/joule/management/usecase/resume"
    params={"name": name, "pod": pod}
    response = requests.request(method="PUT", url=url, params=params)
    if response.status_code != 204:
        displayResponse(name,response)


@click.command()
@click.argument("name")
@click.pass_context
@click.option("--pod", help="Joule pod name", default="")
def pause(ctx: click.Context, name: str, pod: str):
    """Pause use case processing

    NAME is the use case
    """
    joule_host = ctx.obj["joule_host"]
    url = f"http://{joule_host}/joule/management/usecase/pause"
    params={"name": name, "pod": pod}
    response = requests.request(method="PUT", url=url, params=params)
    if response.status_code != 204:
        displayResponse(name,response)


import click
from tabulate import tabulate
from joulectl import joulecomms as jc

def executeDeploy(url: str, filename: str, params: dict, host: str):
    response = jc.postData(url=url,
                           filename=filename,
                           params=params,
                           headers={"Content-Type": "application/json; charset=utf-8"},
                           host=host)
    if response.status_code == 500:
        click.echo("Failed to deploy. Check deployment file {} for syntax errors".format(filename))
        exit(1)
    if params["pod"]:
        data = response.json()
        table = []
        for rsp in data["hostResponses"]:
            cmd = rsp["command"]
            msg = rsp["message"]
            code = rsp["serviceCode"]
        table.append([rsp["host"], cmd, code, msg])
        click.echo(tabulate(table, headers=["Host","Command", "Response", "Message"]))
    else:
       click.echo(response.text)

@click.command()
@click.argument("filename", type=click.Path(exists=True))
@click.option("--pod", help="Joule pod name", default="")
@click.pass_context
def transport(ctx: click.Context, filename: str, pod: str):
    """Deploy transport to the Joule server.

    FILENAME is the specification file to deploy
    """
    joule_host = ctx.obj["joule_host"]
    url = f"http://{joule_host}/joule/management/transports/register"
    options = {'pod':pod}
    executeDeploy(url=url,
                filename=filename,
                params=options,
                host=joule_host)

@click.command()
@click.argument("filename",type=click.Path(exists=True))
@click.option("--pod", help="Joule pod name", default="")
@click.pass_context
def stream(ctx: click.Context, filename: str, pod: str):
    """Deploy a stream to the Joule server.

    FILENAME is the specification file to deploy
    """
    joule_host = ctx.obj["joule_host"]
    url = f"http://{joule_host}/joule/management/stream/register"
    options = {'pod':pod}
    executeDeploy(url=url,
                  filename=filename,
                  params=options,
                  host=joule_host)

@click.command()
@click.argument("filename", type=click.Path(exists=True))
@click.option("--pod", help="Joule pod name", default="")
@click.pass_context
def usecase(ctx: click.Context, filename: str, pod: str):
    """Deploy use case to the Joule server.

    FILENAME is the specification file to deploy
    """
    joule_host = ctx.obj["joule_host"]
    url = f"http://{joule_host}/joule/management/usecase/deploy"
    params = {'pod':pod}
    executeDeploy(url=url,
                  filename=filename,
                  params=params,
                  host=joule_host)
